using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapStoreDataItemTest {
    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructEmpty1()
    {
      ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet());
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructEmpty2()
    {
      ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {});
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructApplicableFlagsEmpty1()
    {
      ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {ImapMessageFlag.Recent, ImapMessageFlag.AllowedCreateKeywords});
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructApplicableFlagsEmpty2()
    {
      ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet(ImapMessageFlag.Recent, ImapMessageFlag.AllowedCreateKeywords));
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructContainsNullKeyword()
    {
      ImapStoreDataItem.ReplaceFlags(new string[] {"$label1", null}, new ImapMessageFlag[] {});
    }

    [Test, ExpectedException(typeof(ArgumentException))]
    public void TestConstructContainsNullFlag()
    {
      ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {ImapMessageFlag.Flagged, null});
    }

    [Test]
    public void TestReplaceFlags()
    {
      Assert.AreEqual("FLAGS (\\Answered \\Deleted)",
                      ImapStoreDataItem.ReplaceFlags(ImapMessageFlag.Answered, ImapMessageFlag.Deleted).ToString());

      Assert.AreEqual("FLAGS ($label1)",
                      ImapStoreDataItem.ReplaceFlags("$label1").ToString());

      Assert.AreEqual("FLAGS (\\Flagged $label3)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label3"}, new[] {ImapMessageFlag.Flagged}).ToString());

      Assert.AreEqual("FLAGS (\\Seen \\Draft $label1 $label2)",
                      ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet(new[] {"$label1", "$label2"}, ImapMessageFlag.Seen, ImapMessageFlag.Draft)).ToString());
    }

    [Test]
    public void TestRemoveNonApplicableFlags1()
    {
      Assert.AreEqual("FLAGS ($label2)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label2"}, new[] {ImapMessageFlag.Recent}).ToString());
    }

    [Test]
    public void TestRejectNonApplicableFlags2()
    {
      Assert.AreEqual("FLAGS ($label2)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label2"}, new[] {ImapMessageFlag.AllowedCreateKeywords}).ToString());
    }

    private void TestInvalidKeyword(string keyword)
    {
      try {
        ImapStoreDataItem.ReplaceFlagsSilent(keyword);
        Assert.Fail("no exception with " + keyword);
      }
      catch (ArgumentException ex) {
        Console.WriteLine(ex.Message);
      }
    }

    [Test]
    public void TestInvalidKeyword()
    {
      TestInvalidKeyword("$la bel1");
      TestInvalidKeyword("$label1\r");
      TestInvalidKeyword("\0$label1\r");
      TestInvalidKeyword("[label1]\r");
      TestInvalidKeyword("(label1)\r");
      TestInvalidKeyword("label:)\r");
      TestInvalidKeyword("label%\r");
      TestInvalidKeyword("label%\r");
    }
  }
}