// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;

namespace Smdn.Net.Imap4 {
  public sealed class ImapBodyDisposition {
    public string Type {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyDictionary<string, string> Parameters {
#else
    public IDictionary<string, string> Parameters {
#endif
      get; private set;
    }

    public bool IsAttachment {
      get { return string.Equals(Type, "attachment", StringComparison.OrdinalIgnoreCase); }
    }

    public bool IsInline {
      get { return string.Equals(Type, "inline", StringComparison.OrdinalIgnoreCase); }
    }

    public string FileName {
      get
      {
        string filename;

        if (Parameters.TryGetValue("filename", out filename))
          return filename;
        else
          return null;
      }
    }

#if NET_4_5
    public ImapBodyDisposition(string type, IReadOnlyDictionary<string, string> parameters)
#else
    public ImapBodyDisposition(string type, IDictionary<string, string> parameters)
#endif
    {
      if (type == null)
        throw new ArgumentNullException("type");

      this.Type = type;
      this.Parameters = ImapBodyStructureUtils.MakeParametersReadOnly(parameters);
    }

    public override string ToString ()
    {
      return string.Format("{{Type={0}, Parameters={1}}}", Type, Parameters.Count);
    }
  }
}
