using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapDateTimeFormatTest {
    [Test]
    public void TestToDateString()
    {
      Assert.AreEqual("1-Feb-1994", ImapDateTimeFormat.ToDateString(new DateTime(1994, 2, 1)));
      Assert.AreEqual("11-Feb-1994", ImapDateTimeFormat.ToDateString(new DateTime(1994, 2, 11)));
      Assert.AreEqual("1-Feb-2000", ImapDateTimeFormat.ToDateString(new DateTime(2000, 2, 1)));
    }

    [Test]
    public void TestToDateTimeString()
    {
      Assert.AreEqual("\"17-Jul-1996 02:44:25 -0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7))));
      Assert.AreEqual("\"07-Jul-1996 02:44:25 -0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7,  7, 2, 44, 25, TimeSpan.FromHours(-7))));
      Assert.AreEqual("\"17-Jul-1996 02:44:25 +0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7))));
      Assert.AreEqual("\"17-Jul-1996 02:44:25 +0000\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero)));
    }

    [Test]
    public void TestFromDateTimeString()
    {
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero),            ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 +0000"));
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero),            ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 -0000"));
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7)),   ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 -0700"));
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7.5)), ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 -0730"));
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7)),   ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 +0700"));
      Assert.AreEqual(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7.5)), ImapDateTimeFormat.FromDateTimeString("17-Jul-1996 02:44:25 +0730"));
    }

    [Test]
    public void TestFromDateTimeStringStartsWithSpace()
    {
      Assert.AreEqual(new DateTimeOffset(2010, 3, 9, 0, 58, 29, TimeSpan.FromHours(-5)),
                      ImapDateTimeFormat.FromDateTimeString(" 9-Mar-2010 00:58:29 -0500"));
    }

    [Test]
    public void TestFromDateTimeStringInvalidFormat()
    {
      Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString("invalid date and time"), "#1");
      if (Runtime.IsRunningOnMono)
        Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString(" 9-mar-2010 00:58:29 -0500"), "#2");
      Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString(" 9-Mar-10 00:58:29 -0500"), "#3");
      Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString("99-Mar-2010 00:58:29 -0500"), "#4");
      Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString("19-Mar-12010 00:58:29 -0500"), "#5");
      Assert.Throws<ImapFormatException>(() => ImapDateTimeFormat.FromDateTimeString(" 9-Mar-2010 00:58:29 0500"), "#6");
    }
  }
}