using System;
using System.Text;
using NUnit.Framework;

using Smdn.Formats;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapEnvelopeTests {
    [Test]
    public void TestGetDecodedSubject()
    {
      var envelope1 = new ImapEnvelope(null, "not encoded", null, null, null, null, null, null, null, null);

      Assert.AreEqual("not encoded", envelope1.GetDecodedSubject());
      Assert.AreEqual("not encoded", envelope1.Subject);

      var envelope2 = new ImapEnvelope(null, "=?ISO-2022-JP?B?GyRCJUYlOSVIGyhC?=", null, null, null, null, null, null, null, null);

      Assert.AreEqual("=?ISO-2022-JP?B?GyRCJUYlOSVIGyhC?=", envelope2.Subject);
      Assert.AreEqual("テスト", envelope2.GetDecodedSubject());

      var envelope3 = new ImapEnvelope(null, null, null, null, null, null, null, null, null, null);

      Assert.IsNull(envelope3.Subject);
      Assert.IsNull(envelope3.GetDecodedSubject());
    }

    [Test]
    public void TestGetDecodedSubjectInvalidMimeEncoding()
    {
      var envelope = new ImapEnvelope(null, "=?ISO-2022-JP?X?GyRCJUYlOSVIGyhC?=", null, null, null, null, null, null, null, null);

      Assert.AreEqual("=?ISO-2022-JP?X?GyRCJUYlOSVIGyhC?=", envelope.Subject);
      Assert.AreEqual("=?ISO-2022-JP?X?GyRCJUYlOSVIGyhC?=", envelope.GetDecodedSubject());
      Assert.AreEqual("=?ISO-2022-JP?X?GyRCJUYlOSVIGyhC?=", envelope.GetDecodedSubject(null, false));

      Assert.Throws<FormatException>(() => envelope.GetDecodedSubject(null, true));
    }

    [Test]
    public void TestGetDecodedSubjectInvalidCharset()
    {
      var envelope = new ImapEnvelope(null, "=?X-UNKNOWN?B?GyRCJUYlOSVIGyhC?=", null, null, null, null, null, null, null, null);
      Assert.AreEqual("=?X-UNKNOWN?B?GyRCJUYlOSVIGyhC?=", envelope.Subject);
      Assert.AreEqual("=?X-UNKNOWN?B?GyRCJUYlOSVIGyhC?=", envelope.GetDecodedSubject());
      Assert.AreEqual("=?X-UNKNOWN?B?GyRCJUYlOSVIGyhC?=", envelope.GetDecodedSubject(null ,false));

      Assert.Throws<EncodingNotSupportedException>(() => envelope.GetDecodedSubject(null, true));

      Assert.AreEqual("テスト", envelope.GetDecodedSubject(delegate(string encoding) {
        Assert.AreEqual("X-UNKNOWN", encoding);

        return Encoding.GetEncoding("iso-2022-jp");
      }, true));
    }
  }
}