using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Diagnostics;
using NUnit.Framework;

namespace Smdn.Net {
  public static class TestUtils {
    public static void SerializeBinary<TSerializable>(TSerializable obj)
      /*where TSerializable : ISerializable*/
    {
      SerializeBinary(obj, null);
    }

    public static void SerializeBinary<TSerializable>(TSerializable obj,
                                                      Action<TSerializable> action)
      /*where TSerializable : ISerializable*/
    {
      var serializeFormatter = new BinaryFormatter();

      using (var stream = new MemoryStream()) {
        serializeFormatter.Serialize(stream, obj);

        stream.Position = 0L;

        var deserializeFormatter = new BinaryFormatter();
        var deserialized = deserializeFormatter.Deserialize(stream);

        Assert.IsNotNull(deserialized);
        Assert.AreNotSame(obj, deserialized);
        Assert.IsInstanceOf<TSerializable>(deserialized);

        if (action != null)
          action((TSerializable)deserialized);
      }
    }

    //private static readonly TimeSpan mergin = TimeSpan.FromTicks(Stopwatch.Frequency);
    private static readonly TimeSpan mergin = TimeSpan.FromMilliseconds(20);

    public static void Elapses(TimeSpan expectedSpan, Action action)
    {
      var sw = Stopwatch.StartNew();

      action();

      sw.Stop();

      Assert.GreaterOrEqual(sw.Elapsed + mergin, expectedSpan);
    }

    public static void NotElapse(TimeSpan expectedSpan, Action action)
    {
      var sw = Stopwatch.StartNew();

      action();

      sw.Stop();

      Assert.LessOrEqual(sw.Elapsed - mergin, expectedSpan);
    }
  }
}
